import copy
import glob
import os
import sys

from ews.coordinator.processor_advisory import ProcessorAdvisory
from ews.coordinator.utils.processor_utils import parse_json_file_with_tokens
from integration.partial.integration_test_utils import IntegrationTestUtils
from integration.test_suites.advisory_test_suite import BaseAdvisoryTestSuite


class FullTestAdvisory(BaseAdvisoryTestSuite.AdvisoryTestSuite):

    def set_expected_values(self):
        self.EXPECTED_EA_IMAGE_COUNT = 6
        self.EXPECTED_ETH_IMAGE_COUNT = 6
        self.EXPECTED_EA_SHAPE_COUNT = 6


    def setUp(self) -> None:

        self.set_expected_values()

        if IntegrationTestUtils.TEST_JOB_DIR is None \
                or not os.path.exists(IntegrationTestUtils.TEST_JOB_DIR):
            # if True:
            FullTestAdvisory.write_temp_run_config_files()
            FullTestAdvisory.run_advisory_pipeline()
        else:
            string = f"output in {IntegrationTestUtils.TEST_JOB_DIR} already written, skipping rerun"
            print(string)
            i = 0


    @staticmethod
    def write_temp_run_config_files():

        """
        IntegrationTestUtils.RUN_SYS_CONFIG_FILE_PATH will have been set from the command line args, and point to the
        production sys config file. We need to modify this file to point to the test output directory.
        :return:
        """
        sys_config = IntegrationTestUtils.RUN_SYS_CONFIG_FILE_PATH
        sys_config_dict: dict = parse_json_file_with_tokens(sys_config)

        sys_config_dict['WorkspacePathout'] = IntegrationTestUtils.TEST_OUT_PATH
        sys_config_dict['WorkspacePath'] = IntegrationTestUtils.TEST_OUT_PATH

        os.makedirs(IntegrationTestUtils.TEST_OUT_PATH, exist_ok = True)
        job_sys_config_file = IntegrationTestUtils.TEST_OUT_PATH + "temp_sys_advisory_config.json"
        IntegrationTestUtils.write_json_file(sys_config_dict, job_sys_config_file)

        # now set the sys config file path to the new job-specific file
        IntegrationTestUtils.RUN_SYS_CONFIG_FILE_PATH = job_sys_config_file

        run_config = IntegrationTestUtils.RUN_CONFIG_FILE_PATH
        run_config_temp_dict: dict = parse_json_file_with_tokens(run_config, sys_config_dict)
        run_dict: dict = copy.deepcopy(run_config_temp_dict)

        job_run_config_file = IntegrationTestUtils.TEST_OUT_PATH + "temp_advisory_config.json"
        IntegrationTestUtils.write_json_file(run_dict, job_run_config_file)

        # set the run config file path to the new job-specific file
        IntegrationTestUtils.RUN_CONFIG_FILE_PATH = job_run_config_file

    @staticmethod
    def run_advisory_pipeline():
        advisory_processor = ProcessorAdvisory()
        IntegrationTestUtils.run_external_pipeline(BaseAdvisoryTestSuite.AdvisoryTestSuite.ADVISORY_COMPONENT_NAME,
                                                   BaseAdvisoryTestSuite.AdvisoryTestSuite.ADVISORY_PROCESSOR_DIR,
                                                   IntegrationTestUtils.TEST_START_DATE,
                                                   advisory_processor)

    def test_standard_run_input_image_files_found(self):
        """
        The advisory will continue and succeed when image outputs from env_suit, depo and epi are not present, a log
        message with status WARNING and prefix: <image_type>_FILE_NOT_FOUND.

        In the test environment, we can safely expect that there will not be any inputs missing, so we can check for
        these log entries and fail if they are present.

        We want to run this test as changes to output filenames in env_suit, depo and epi can result in the advisory
        quietly missing these new filenames, allowing us to catch them in the full integration test before deployment

        :return:
        """
        log_file_path = os.path.join(IntegrationTestUtils.TEST_OUT_PATH,
                                     IntegrationTestUtils.TEST_JOB_DIR,
                                     "log.txt")

        self.assertTrue(os.path.isfile(log_file_path))
        fig_not_found_count = IntegrationTestUtils.count_tokens_in_file(log_file_path, "FIGURE_NOT_FOUND")

        self.assertEqual(0, fig_not_found_count)


if __name__ == '__main__':
    _success: bool = IntegrationTestUtils.run_full_integration_test_pipeline(FullTestAdvisory,
                                                                             test_prefix = "advisory",
                                                                             processor_dir = BaseAdvisoryTestSuite.AdvisoryTestSuite.ADVISORY_PROCESSOR_DIR)
    if not _success:
        sys.exit(1)
